﻿#***************************************************************************************# 
#                                                                                       #
#        #### #### #   # #### #### #### #  # #### #    #    ###  #### ##  # ####        #
#        #  # #  # # # # #    #  # #    #  # #    #    #    #  # #  # # # # #           #
#        #### #  # # # # ###  #### #### #### ###  #    #    #  # #  # # # # ###         #
#        #    #  # # # # #    # #     # #  # #    #    #    #  # #  # #  ## #           #
#        #    ####  # #  #### #  # #### #  # #### #### #### ###  #### #  ## ####        #
#                                                                                       #
#***************************************************************************************#
#                                                                    #
# Name : Get-ACLFromFolder                                           #
# Date : 12/11/2013                                                  #
# Author : Eric Fernandes                                            #
# M@il : fernandes_eric.hotmail.fr                                   #
# Subject : The script extract all NTFS permissions from a folder    # 
#           and sub-folder on an excel file.                         # 
#           The script scan every ACL s groups and provide members.  #
#                                                                    #
#********************************************************************#


Function Get-ACLFromFolder
{
    [CmdletBinding()]
    param(
        [parameter(Mandatory=$true,ValueFromPipeline=$False)]
        [String]$Target,
        [parameter(Mandatory=$true,ValueFromPipeline=$False)]
        [String]$Level
    )

    Begin
    {
        #Test the path Target
        $TstPath = $(try{Get-ChildItem $Target -ErrorAction SilentlyContinue}catch{$null})
        if($TstPath -eq $null)
        {
            #popup
            $WshShell = New-Object -ComObject Wscript.Shell
            $WshShell.popup("Error when the script try to reach the path $Target ! Please try again.")
            Exit
        }
        else
        {
            $TstACL = $(try{Get-Acl $Target -ErrorAction SilentlyContinue}catch{$null})
            if($TstACL -eq $null)
            {
                #popup
                $WshShell = New-Object -ComObject Wscript.Shell
                $WshShell.popup("No access to the path $Target ! Please try again.")
                Exit
            }
        }

        $Date= Get-Date -Format 'yyyyMMdd'
        $npath = ($Target -replace '^.*.\\','')+'_'+'LVL'+$Level+'_'+$Date |echo

        # Creation du fichier Excel

        $ObjExcel = New-Object -ComObject Excel.application
        $WorkBook = $ObjExcel.Workbooks.add()
        $WorkBook
        $WorkSheet1 = $WorkBook.WorkSheets.item(1)
        $WorkSheet2 = $WorkBook.WorkSheets.item(2)
        $WorkBook.WorkSheets.Item(1).Name = 'ACLAndUsersFromFolders'
        $WorkBook.WorkSheets.Item(2).Name = 'MembersOfGroups'
        if($PSCulture -eq 'fr-FR')
        {
            $S3 = $objexcel.sheets | where {$_.name -eq "Feuil3"}
        }
        else
        {
            $S3 = $objexcel.sheets | where {$_.name -eq "Sheet3"}
        }
        
        $S3.delete() 

        $Ligne = 2
        $ObjExcel.Cells.Item($Ligne,2) = "GET-ACLFROMFOLDER" 
        $ObjExcel.Cells.Item($Ligne,2).Font.Bold = $True 
        $ObjExcel.Cells.Item($Ligne,2).Font.size = 20
        $ObjExcel.Cells.Item($Ligne,2).Font.color = 15123099
        $Ligne += 2

        $File = "C:\Users\USERNAME\Desktop\AclFromFolder\Result\ACLFromFolder-$npath"

        

        $AllItem = for($i=1; $i -le $Level; $i++)
                    {
                        Switch($i)
                        {
                            1 {$AntiSlashEtoile = '\*'}
                            2 {$AntiSlashEtoile = '\*\*'}
                            3 {$AntiSlashEtoile = '\*\*\*'}
                            4 {$AntiSlashEtoile = '\*\*\*\*'}
                            5 {$AntiSlashEtoile = '\*\*\*\*\*'}
                            6 {$AntiSlashEtoile = '\*\*\*\*\*\*'}
                            7 {$AntiSlashEtoile = '\*\*\*\*\*\*\*'}
                            8 {$AntiSlashEtoile = '\*\*\*\*\*\*\*\*'}
                            9 {$AntiSlashEtoile = '\*\*\*\*\*\*\*\*\*'}
                            10 {$AntiSlashEtoile = '\*\*\*\*\*\*\*\*\*\*'}
                            Default {$AntiSlashEtoile = '\*'}
                        }
                        Get-ChildItem "$Target$AntiSlashEtoile" -Attributes Directory           
                    }

        $NbrFolder=$AllItem | measure
        $NbrFolder = $NbrFolder.Count 

        # Infos sur le repertoire principale

        $WorkSheet1.Cells.Item($Ligne,1) = 'Principal Folder :' 
        $WorkSheet1.Cells.Item($Ligne,1).Font.Bold = $True 
        $WorkSheet1.Cells.Item($Ligne,1).Interior.ColorIndex = 15
        $WorkSheet1.Cells.Item($Ligne,2) = "Groups :" 
        $WorkSheet1.Cells.Item($Ligne,2).Font.Bold = $True
        $WorkSheet1.Cells.Item($Ligne,2).Interior.ColorIndex = 15
        $WorkSheet1.Cells.Item($Ligne,3) = "Rights :" 
        $WorkSheet1.Cells.Item($Ligne,3).Font.Bold = $True
        $WorkSheet1.Cells.Item($Ligne,3).Interior.ColorIndex = 15

        $Ligne++

        $ObjACLs = get-acl $Target
        $RName = $($ObjACLs.Path -replace 'Microsoft.PowerShell.Core\\FileSystem::','')
    }
    Process
    {
        $WorkSheet1.Cells.Item($Ligne,1) = $RName
        $WorkSheet1.Cells.Item($Ligne,1).Font.Bold = $True 
        $WorkSheet1.Cells.Item($Ligne,1).Interior.ColorIndex = 3
        $Ligne++ 
        #Groups from principal folder
        foreach($ObjACL in $ObjACLs)
        {
            $IR = $ObjACL.Access.identityreference
            $Right = $ObjACL.Access.FileSystemRights
    
            $NbrGrp = $IR | measure
            $NbrGrp = $NbrGrp.Count
    
            for($i=0; $i -lt $NbrGrp; $i++)
            {
                [String]$IdRef = $IR[$i]
                $IRFrom = $IdRef -replace '^.*.\\',''
                $Prefix = $IdRef -replace '\\.*.$',''
                $AccessRight = $Right[$i]
                
                if($AccessRight -eq '1179817')
                {
                    $AccessRight = 'Read and execute'
                }
                elseif($AccessRight -eq '1245631')
                {
                    $AccessRight = 'Modify'
                }
                elseif($AccessRight -eq '2032127')
                {
                    $AccessRight = 'Full'
                }
                elseif($AccessRight -eq '1048609')
                {
                    $AccessRight = 'Traverse Folder'
                }
                else
                {
                    $AccessRight = 'Special'
                }
                        
                $TestGroupABT = $(try{Get-ADGroup -Identity $IRFrom}catch{$null})
                if ($TestGroupABT -ne $null) # If it's an NTFS group
                {
                    $WorkSheet1.Cells.Item($Ligne,2) = $IdRef
                    $WorkSheet1.Cells.Item($Ligne,2).Interior.ColorIndex = 37
                    $WorkSheet1.Cells.Item($Ligne,3) = $AccessRight
                    $OldRNameABT = $RName
                    $Ligne++ 
                        
                    [String[]]$ListGroup += $IRFrom
                }
                else # If it's not an NTFS group (User or Local Group)
                {
                    $WorkSheet1.Cells.Item($Ligne,2) = $IdRef
                    $WorkSheet1.Cells.Item($Ligne,2).Interior.ColorIndex = 10
                    $WorkSheet1.Cells.Item($Ligne,3) = $AccessRight
                    $Ligne++
                }
            }
        }

        $Ligne = $Ligne + 2

        #Groups from sub-folder

        $WorkSheet1.Cells.Item($Ligne,1) = "There is $NbrFolder sub-folders with the scan level $Level :"
        $WorkSheet1.Cells.Item($Ligne,1).Font.Bold = $True 
        $WorkSheet1.Cells.Item($Ligne,1).Interior.ColorIndex = 15
        $Ligne++

        $WorkSheet1.Cells.Item($Ligne,1) = "Sub-Folders :"
        $WorkSheet1.Cells.Item($Ligne,1).Font.Bold = $True 
        $WorkSheet1.Cells.Item($Ligne,1).Interior.ColorIndex = 15
        $WorkSheet1.Cells.Item($Ligne,2) = "Groups :" 
        $WorkSheet1.Cells.Item($Ligne,2).Font.Bold = $True
        $WorkSheet1.Cells.Item($Ligne,2).Interior.ColorIndex = 15
        $WorkSheet1.Cells.Item($Ligne,3) = "Rights :" 
        $WorkSheet1.Cells.Item($Ligne,3).Font.Bold = $True
        $WorkSheet1.Cells.Item($Ligne,3).Interior.ColorIndex = 15
        $Ligne++

        $Cpt = 1 
        
        foreach($Folder in $AllItem)
        {
            Write-Progress -Activity "Get-ACLFromFolder" -status "Sub-Folder $Cpt/$NbrFolder" -percentComplete ($Cpt/$NbrFolder*100) -Id 1   
            $ObjACLs = $Folder | Get-Acl
            $RName = $($ObjACLs.Path -replace 'Microsoft.PowerShell.Core\\FileSystem::','')
            
            $WorkSheet1.Cells.Item($Ligne,1) = $RName
            $WorkSheet1.Cells.Item($Ligne,1).Font.Bold = $True 
            $WorkSheet1.Cells.Item($Ligne,1).Interior.ColorIndex = 45
            $Ligne++ 

            foreach ($ObjACL in $ObjACLs)
            {
                $IR = $ObjACL.Access.identityreference
                $Right = $ObjACL.Access.FileSystemRights
    
                $NbrGrp = $IR | measure
                $NbrGrp = $NbrGrp.Count

                for($i=0; $i -lt $NbrGrp; $i++)
                {
                    [String]$IdRef = $IR[$i]
                    $IRFrom = $IdRef -replace '^.*.\\',''
                    $Prefix = $IdRef -replace '\\.*.$',''
                    
                    $AccessRight = $Right[$i]

                    if($AccessRight -eq '1179817')
                    {
                        $AccessRight = 'Read and execute'
                    }
                    elseif($AccessRight -eq '1245631')
                    {
                        $AccessRight = 'Modify'
                    }
                    elseif($AccessRight -eq '2032127')
                    {
                        $AccessRight = 'Full'
                    }
                    elseif($AccessRight -eq '1048609')
                    {
                        $AccessRight = 'Traverse Folder'
                    }
                    else
                    {
                        $AccessRight = 'Special'
                    }
                        
                    $TestGroupABT = $(try{Get-ADGroup -Identity $IRFrom}catch{$null})
                    if ($TestGroupABT -ne $null) # If it's an NTFS group
                    {
                        $WorkSheet1.Cells.Item($Ligne,2) = $IdRef 
                        $WorkSheet1.Cells.Item($Ligne,2).Interior.ColorIndex = 37
                        $WorkSheet1.Cells.Item($Ligne,3) = $AccessRight
                        if($IRFrom -notin $ListGroup)
                        {
                            $ListGroup += $IRFrom
                        }
                        $Ligne++                
                    }
                    else # If it's not a group (user)
                    {
                        $WorkSheet1.Cells.Item($Ligne,2) = $IdRef 
                        $WorkSheet1.Cells.Item($Ligne,2).Interior.ColorIndex = 10
                        $WorkSheet1.Cells.Item($Ligne,3) = $AccessRight
                        $Ligne++   
                    }
                }
            } 
            $Cpt++               
        }
        
        $LigneSheet2 = 1

        $WorkSheet2.Cells.Item($LigneSheet2,1) = 'ACL Groups'
        $WorkSheet2.Cells.Item($LigneSheet2,1).Font.Bold = $True 
        $WorkSheet2.Cells.Item($LigneSheet2,1).Interior.ColorIndex = 15
        $WorkSheet2.Cells.Item($LigneSheet2,2) = 'SamAccountName' 
        $WorkSheet2.Cells.Item($LigneSheet2,2).Font.Bold = $True
        $WorkSheet2.Cells.Item($LigneSheet2,2).Interior.ColorIndex = 15
        $WorkSheet2.Cells.Item($LigneSheet2,3) = 'First Name' 
        $WorkSheet2.Cells.Item($LigneSheet2,3).Font.Bold = $True
        $WorkSheet2.Cells.Item($LigneSheet2,3).Interior.ColorIndex = 15
        $WorkSheet2.Cells.Item($LigneSheet2,4) = 'Last Name'
        $WorkSheet2.Cells.Item($LigneSheet2,4).Font.Bold = $True 
        $WorkSheet2.Cells.Item($LigneSheet2,4).Interior.ColorIndex = 15
        $WorkSheet2.Cells.Item($LigneSheet2,5) = 'Mail' 
        $WorkSheet2.Cells.Item($LigneSheet2,5).Font.Bold = $True
        $WorkSheet2.Cells.Item($LigneSheet2,5).Interior.ColorIndex = 15
        $WorkSheet2.Cells.Item($LigneSheet2,6) = 'Enabled' 
        $WorkSheet2.Cells.Item($LigneSheet2,6).Font.Bold = $True
        $WorkSheet2.Cells.Item($LigneSheet2,6).Interior.ColorIndex = 15
        
        $LigneSheet2++
        $NbrGroup = $ListGroup | measure
        $NbrGroup = $NbrGroup.Count
        $Cpt2 = 1 
       
        foreach ($Group in $ListGroup)
        {
            Write-Progress -Activity "Get-ACLFromFolder" -status "Group $Cpt2/$NbrGroup" -percentComplete ($Cpt2/$NbrGroup*100) -Id 2   
            $WorkSheet2.Cells.Item($LigneSheet2,1) = "$Group"
            $WorkSheet2.Cells.Item($LigneSheet2,1).Font.Bold = $True 
            $WorkSheet2.Cells.Item($LigneSheet2,1).Interior.ColorIndex = 37
            $LigneSheet2++

            $Members = Get-ADGroupMember -Identity $Group -recursive
            $Desc = Get-ADGroup -Identity $Group -Properties Description
            $Description = $Desc.Description
            
            $WorkSheet2.Cells.Item($LigneSheet2,1) = "Desc att: $Description"
            $WorkSheet2.Cells.Item($LigneSheet2,1).Font.Bold = $True 
            $WorkSheet2.Cells.Item($LigneSheet2,1).Interior.ColorIndex = 34

            $LigneSheet2++

            $NbrMember = $Members | measure
            $NbrMember = $NbrMember.Count
            $Cpt3 = 1
            
            foreach ($Element in $Members)
            { 
                Write-Progress -Activity "Get-ACLFromFolder" -status "Member $Cpt3/$NbrMember" -percentComplete ($Cpt3/$NbrMember*100) -Id 3      
                if ($Element.objectClass -eq 'user')
                {  
                    $User = $(try{Get-ADUser -Identity $Element -Properties GivenName,Surname,Mail,enabled -ErrorAction SilentlyContinue}catch{$null})
                    if($User -ne $null)
                    {
                        $SamAccountName = $User.SamAccountName
                        $GivenName = $User.GivenName 
                        $SurName = $User.Surname
                        $Mail = $User.Mail
                        $Enabled = $User.Enabled

                        $WorkSheet2.Cells.Item($LigneSheet2,2) = $SamAccountName
                        $WorkSheet2.Cells.Item($LigneSheet2,2).Interior.ColorIndex = 37
                        $WorkSheet2.Cells.Item($LigneSheet2,3) = $GivenName
                        $WorkSheet2.Cells.Item($LigneSheet2,3).Interior.ColorIndex = 37
                        $WorkSheet2.Cells.Item($LigneSheet2,4) = $SurName
                        $WorkSheet2.Cells.Item($LigneSheet2,4).Interior.ColorIndex = 37
                        $WorkSheet2.Cells.Item($LigneSheet2,5) = $Mail
                        $WorkSheet2.Cells.Item($LigneSheet2,5).Interior.ColorIndex = 37
                        $WorkSheet2.Cells.Item($LigneSheet2,6) = $Enabled
                        $WorkSheet2.Cells.Item($LigneSheet2,6).Interior.ColorIndex = 37
                        $LigneSheet2++
                    }
                }
                elseif($Element.objectClass -eq 'group')
                {
                    $User = Get-ADGroup -Identity $Element -Properties Name
                    $Name = $User.Name
                    $WorkSheet2.Cells.Item($LigneSheet2,2) = "**$Name**"
                    $LigneSheet2++
                } 
                $Cpt3++              
            } 
            $Cpt2++
        }
    }
    End
    {
        $Ligne++ 

        # Enregistrement du fichier Excel

        $Ligne = $Ligne - 2
        $LigneSheet2--
        $Range1 = $WorkSheet1.Range("A4","C$Ligne")
        $ligne += 2
        $ObjExcel.Cells.Item($Ligne,2) = "#Export made by PoWerShellDoNe" 
        $ObjExcel.Cells.Item($Ligne,2).Font.Bold = $True 
        $ObjExcel.Cells.Item($Ligne,2).Font.size = 14
        $ObjExcel.Cells.Item($Ligne,2).Font.color = 15123099
        
        $Range1.Borders.Weight = 2
        $Range2 = $WorkSheet2.Range("A1","F$LigneSheet2")
        $LigneSheet2 += 2
        $WorkSheet2.Cells.Item($LigneSheet2,5) = "#Export made by PoWerShellDoNe" 
        $WorkSheet2.Cells.Item($LigneSheet2,5).Font.Bold = $True 
        $WorkSheet2.Cells.Item($LigneSheet2,5).Font.size = 14
        $WorkSheet2.Cells.Item($LigneSheet2,5).Font.color = 15123099
        
        $Range2.Borders.Weight = 2
        $WorkSheet1.Columns.autofit()
        $WorkSheet2.Columns.autofit()
        $ObjExcel.ActiveWorkbook._SaveAs($File)
        $ObjExcel.WorkBooks.close()
        $ObjExcel.Quit()
        [System.Runtime.Interopservices.Marshal]::ReleaseComObject($objExcel)
    }
}

#****************************************************************#
#                                                                #
#              Test PowerShell Environment                       #
#                                                                #
#****************************************************************#

$TstVersionPS = $PSVersionTable.PSVersion.Major
if($TstVersionPS -ge '3')
{
    $TstExecutionPolicy = Get-ExecutionPolicy
    if(($TstExecutionPolicy -eq 'RemoteSigned') -or ($TstExecutionPolicy -eq 'Unrestricted'))
    {
        $TstModuleAD = $(try{Get-Module -ListAvailable ActiveDirectory -ErrorAction SilentlyContinue}catch{$null})
        if($TstModuleAD -ne $null)
        {
            
        }
        else
        {
            #popup for issue
            $WshShell = New-Object -ComObject Wscript.Shell
            $WshShell.popup("You need the module ActiveDirectory !")
            Exit
        }
    }
    else
    {
        #popup for issue
        $WshShell = New-Object -ComObject Wscript.Shell
        $WshShell.popup("You need to set the Execution Policy of your machine to RemoteSigned or Unrestricted ! Currently it's set to $TstExecutionPolicy")
        Exit
    }
}
else
{
    #popup for issue
    $WshShell = New-Object -ComObject Wscript.Shell
    $WshShell.popup("You need at less the version 3 of PowerShell ! Currently you have the version $TstVersionPS")
    Exit
}

#****************************************************************#
#                                                                #
#             End Test PowerShell Environment                    #
#                                                                #
#****************************************************************#

#Assembly for WPF
Add-Type -AssemblyName PresentationFramework
Add-Type -AssemblyName PresentationCore
Add-Type -AssemblyName WindowsBase

#****************************************************************#
#                                                                #
#                      Start Step 1 Action                       #
#                                                                #
#****************************************************************#

#Path of the XAML's file for WPF Target
[xml]$XAMLTarget = Get-Content "C:\Users\USERNAME\Desktop\AclFromFolder\Get-ACLFromFolder.xaml"
$ReaderTarget=New-Object System.Xml.XmlNodeReader $XAMLTarget
$FormTarget=[Windows.Markup.XamlReader]::Load($ReaderTarget)

$LaunchAccount = $FormTarget.FindName('LaunchAccount')
$LaunchAccount.Items.Add($env:USERNAME)
$BtnNextTarget = $FormTarget.FindName('Launch')
$BtnQuitTarget = $FormTarget.FindName('Quit')

$BtnNextTarget.add_click({

    $Level=$FormTarget.FindName('Level')
    $ScanFolder=$FormTarget.FindName('Folder')
    $ScanFolder=$ScanFolder.Text
    $Level = $Level.Text
    $FormTarget.close()
    Get-ACLFromFolder -Target $ScanFolder -Level $Level

    $WshShell = New-Object -ComObject Wscript.Shell
    $WshShell.popup("Check your report on C:\Users\USERNAME\Desktop\AclFromFolder\Result !")


})

$BtnQuitTarget.add_click({

    $FormTarget.close()

})
$FormTarget.Showdialog() | Out-Null